function tehat = teendbin(par,y,x,z)
    %% Efficiency scores for the SFM with endogenous treatment
    % Centorrino et al. (2023, Journal of Applied Econometrics)
    % Number of observations
    N         = length(y);
    
    % Arguments:
    % par = vector of estimated parameters from likendsftreat.m
    % y   = Nx1 vector, dependent variable 
    % x   = N x p matrix, inputs (not including the constant term)
    % z   = N x q matrix, environmental variables
    % Author: Samuele Centorrino
    % Email: samuele.centorrino@stonybrook.edu

    betahat   = par(1:(size(x,2)+1));
    deltahat  = par((size(x,2) + 3):(size(x,2) + size(z,2) + 2));
    rhovhat   = par(size(x,2) + size(z,2) + 3);
    rhouhat   = par(size(x,2) + size(z,2) + 4);
    sv2hat    = par(size(x,2) + size(z,2) + 5);
    su2hat    = exp(2*par(size(x,2) + 2));
    %gammahatres  = par((size(x,2) + size(z,2) + 6):(size(x,2) + size(z,2) + 5 + size(EXV,2)));

    tsigmav   = sqrt((1 - rhovhat^2)*sv2hat);
    tsigmau   = exp(par(size(x,2) + 1) + z*deltahat);
    lambda    = real(tsigmau./tsigmav);
    sigma2    = tsigmav.^2 + tsigmau.^2;
    sigma     = sqrt(sigma2);

    eps       = y - [ones(N,1) x]*betahat;
    
    sigmastar = (tsigmav.*tsigmau)./(sigma);
    
    % FIRST COMPONENT OF MIXTURE
    sigmau  = sqrt(su2hat).*exp(z*deltahat);
    qp      = lambda.*rhovhat*sqrt(sv2hat)./sigma + (rhouhat.*sigmau)./(lambda.*sigma);
    rhop    = rhovhat*sqrt(sv2hat) - rhouhat.*sqrt(su2hat).*exp(z*deltahat);
    
    mustarp = sigmastar.*(-lambda./sigma + qp.*rhop./(sigma2 + rhop.^2)).*eps;
    sigmastarp = sigmastar.*sqrt(1 + (qp.^2).*sigma2./(sigma2 + rhop.^2));
    taup    = (-lambda./sigma + qp.*rhop./(sigma2 + rhop.^2))./sqrt(1 + (qp.^2).*sigma2./(sigma2 + rhop.^2));
    
    cstep   = exp(-mustarp + (sigmastarp.^2)/2);
    
    % SECOND COMPONENT OF MIXTURE
    qn      = lambda.*rhovhat*sqrt(sv2hat)./sigma - ...
              (rhouhat.*sqrt(su2hat).*exp(z*deltahat))./(lambda.*sigma);
    rhon    = rhovhat*sqrt(sv2hat) + rhouhat.*sqrt(su2hat).*exp(z*deltahat);
    
    mustarn    = sigmastar.*(-lambda./sigma + qn.*rhon./(sigma2 + rhon.^2)).*eps;
    sigmastarn = sigmastar.*sqrt(1 + (qn.^2).*sigma2./(sigma2 + rhon.^2));
    taun      = (-lambda./sigma + qn.*rhon./(sigma2 + rhon.^2))./sqrt(1 + (qn.^2).*sigma2./(sigma2 + rhon.^2));
    
    cstenres      = exp(-mustarn + (sigmastarn.^2)/2);
    
    normfactp  = normcdf(mustarp./sigmastarp - sigmastarp)./normcdf(taup.*eps);
    normfactn  = normcdf(mustarn./sigmastarn - sigmastarn)./normcdf(taun.*eps);
    
    normfactp(isnan(normfactp)) = 0;
    normfactn(isnan(normfactn)) = 0;
    
    fepsp      = normcdf(taup.*eps).*normpdf(eps,0,sqrt((sigma2 + rhop.^2)));
    fepsn      = normcdf(taun.*eps).*normpdf(eps,0,sqrt((sigma2 + rhon.^2)));
        
    tehat     = (fepsp.*cstep.*normfactp + fepsn.*cstenres.*normfactn)./(fepsp + fepsn);

end
