function tehat = teex(par,y,x,z)
    %% Efficiency scores for the normal-half-normal SFM
    %  with linear frontier and scale exponential inefficiency
    %  Number of observations
    N            = length(y);
    
    % Arguments:
    % par = vector of estimated parameters from liksf.m
    % y   = Nx1 vector, dependent variable 
    % x   = N x p matrix, inputs (not including the constant term)
    % z   = N x q matrix, environmental variables
    % Author: Samuele Centorrino
    % Email: samuele.centorrino@stonybrook.edu

    totsize      = size(x,2) + size(z,2);
    deltahatex   = par((size(x,2)+2):(totsize + 1));
    sv2hatex     = par(totsize + 2);
    su2hatex     = par(totsize + 3);
    betahatex    = par(1:(size(x,2)+1));
    
    eps          = y - [ones(N,1) x]*betahatex;

    % Technical inefficiency using Battese-Coelli formula
    mustar       = -eps.*(su2hatex.*exp(2*z*deltahatex))./(sv2hatex + su2hatex.*exp(2*z*deltahatex));
    sigmastar    = sqrt((su2hatex.*exp(2*z*deltahatex).*sv2hatex)./(sv2hatex + su2hatex.*exp(2*z*deltahatex)));
    
    normfact     = normcdf(mustar./sigmastar - sigmastar)./normcdf(mustar./sigmastar);
    normfact(isnan(normfact)) = 0;
    
    tehat    = normfact.*exp(-mustar + (sigmastar.^2)/2);

end
